<?php

namespace TdyGames\Table;

use TdyGames\Model;
use TdyCommons\Table\Table;
use TdyPlayers\Model\Player;
use Zend\Db\ResultSet\HydratingResultSet;
use Zend\Db\Sql\Delete;
use Zend\Db\Sql\Insert;
use Zend\Db\Sql\Predicate;
use Zend\Db\Sql\Select;
use Zend\Db\Sql\Sql;
use Zend\Db\Sql\Update;
use Zend\Db\Sql\Where;
use Zend\Paginator\Adapter\DbSelect;
use Zend\Paginator\Paginator;

/**
 * Class Slots
 *
 * @package TdyGames\Table
 *
 * @author  James Lloyd Atwil <james@nadows.com>
 */
class Slots extends Table
{

    /**
     * @var array
     */
    public $relatedTables = [];

    /**
     * @param array $relatedTables
     */
    public function setRelatedTables($relatedTables)
    {
        $this->relatedTables = $relatedTables;
    }

    /**
     * @param int    $cn
     * @param string $q
     * @param array  $exclusion
     * @param array  $filters
     * @param array  $sort
     * @param bool   $paginated
     *
     * @return bool|HydratingResultSet|Paginator
     */
    public function fetch($cn, $q = '', $exclusion = [], $filters = [], $sort = [], $paginated = false)
    {
        $this->getLogger()->debug($this->indent . 'Contests\Slots::fetch() start.');

        $select = new Select();
        $select->from($this->getTableGateway()->getTable());
        $conditions = [];

        $where = new Where([]);
        $where->addPredicate(new Predicate\Expression($this->columns['contest-id'] . ' = ?', $cn));
        $conditions[] = $where;

        $where = new Where([]);
        $where->addPredicate(new Predicate\Expression($this->columns['is-active'] . ' = ?', 1));
        $conditions[] = $where;

        if (!empty($q)) {
            $where = new Where([], Predicate\PredicateSet::OP_OR);
            $where->addPredicate(new Predicate\Like($this->columns['slug'], '%' . $q . '%'));
            $where->addPredicate(new Predicate\Like($this->columns['name'], '%' . $q . '%'));
            $conditions[] = $where;
        }

        if (!empty($filters)) {
            $where = new Where([]);
            foreach ($filters as $k => $v) {
                if (is_array($v)) {
                    $where->addPredicate(new Predicate\In($k, $v));
                } else {
                    $where->addPredicate(new Predicate\Expression($k . ' = ?', $v));
                }
            }
            $conditions[] = $where;
        }

        if (!empty($exclusion)) {
            $where = new Where([]);
            $where->addPredicate(new Predicate\NotIn('id', $exclusion));
            $conditions[] = $where;
        }

        $select->where($conditions, Predicate\PredicateSet::OP_AND);

        if (!is_null($sort)) {
            if ($this->isSortEmpty($sort)) {
                $select->order([
                    $this->columns['id'] => 'DESC',
                ]);
            } else {
                $select->order($sort);
            }
        } else {
            $select->order($sort);
        }

        $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'SQL Statement: ' . $select->getSqlString($this->getAdapter()->getPlatform()));

        $resultSet = new HydratingResultSet();
        $resultSet->setObjectPrototype(new Model\Slot());

        if ($paginated) {
            $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'Is Paginable: YES');

            $paginatorAdapter = new DbSelect($select, $this->getTableGateway()->getSql(), $resultSet);
            $paginator        = new Paginator($paginatorAdapter);

            $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'Returns Paginator instance.');
            $this->getLogger()->debug($this->indent . 'Contests\Slots::fetch() end.');

            return $paginator;
        } else {
            $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'Is Paginable: NO');

            $sql       = new Sql($this->tableGateway->getAdapter());
            $statement = $sql->prepareStatementForSqlObject($select);

            $results = $statement->execute();
            $resultSet->initialize($results);

            if (!$resultSet) {
                $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'Returns FALSE');
                $this->getLogger()->debug($this->indent . 'Contests\Slots::fetch() end.');

                return false;
            } else {
                $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'Returns HydratingResultSet instance.');
                $this->getLogger()->debug($this->indent . 'Contests\Slots::fetch() end.');

                return $resultSet;
            }
        }
    }

    /**
     * Returns the Slot.
     *
     * @param int $id
     *
     * @return Model\Slot|null
     * @throws \Exception
     */
    public function get($id)
    {
        $rowset = $this->tableGateway->select([$this->columns['id'] => $id]);
        $row    = $rowset->current();
        if (!$row) {
            return null;
        }

        return $row;
    }

    /**
     * @param Model\Slot $slot
     *
     * @throws \Exception
     */
    public function save(Model\Slot $slot)
    {
        $this->getLogger()->debug($this->indent . 'Slots\Slots::save() start.');

        $data = [
            $this->columns['id']      => $slot->id,
            $this->columns['player-id']    => $slot->playerId,
            $this->columns['total-points'] => $slot->totalPoints,
        ];

        $id = (int) $slot->id;
        if ($id == 0) {
            $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'Inserting Slot record.');

            $data['is_active']  = 1;
            $data['created_by'] = $slot->createdBy;
            $data['created_on'] = date('Y-m-d H:i:s');

            $insert = new Insert($this->getTableGateway()->getTable());
            $insert->values($data);

            $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'SQL Statement: ' . $insert->getSqlString($this->getAdapter()->getPlatform()));

            $sql       = new Sql($this->tableGateway->getAdapter());
            $statement = $sql->prepareStatementForSqlObject($insert);

            $statement->execute();

            $id = $this->getAdapter()->getDriver()->getLastGeneratedValue();

            $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'Slot ' . $id . ' has been inserted.');
            $this->getLogger()->debug($this->indent . 'Slots\Slots::save() end.');

            return $this->get($id);
        } else {
            if ($this->get($id)) {
                $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'Updating Slot record.');

                $data[$this->columns['updated-by']] = $slot->updatedBy;
                $data[$this->columns['updated-on']] = date('Y-m-d H:i:s');

                $update = new Update($this->getTableGateway()->getTable());
                $update->set($data);
                $update->where([$this->columns['id'] => $id]);

                $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'SQL Statement: ' . $update->getSqlString($this->getAdapter()->getPlatform()));

                $sql       = new Sql($this->tableGateway->getAdapter());
                $statement = $sql->prepareStatementForSqlObject($update);

                $statement->execute();

                $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'Slot ' . $id . ' has been updated.');
                $this->getLogger()->debug($this->indent . 'Slots\Slots::save() end.');

                return $this->get($id);
            } else {
                $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'Slot ID ' . $id . ' does not exists. Update failed.');
                $this->getLogger()->debug($this->indent . 'Slots\Slots::save() end.');

                throw new \Exception('Slot ID does not exist');
            }
        }
    }

    public function setPlace(Model\Slot $slot)
    {
        $this->getLogger()->debug($this->indent . 'Slots\Slots::savePlace() start.');

        $data = [
            $this->columns['place'] => $slot->place,
        ];

        $id = (int) $slot->id;

        if ($this->get($id)) {
            $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'Setting place on the Slot.');

            $data[$this->columns['updated-by']] = $slot->updatedBy;
            $data[$this->columns['updated-on']] = date('Y-m-d H:i:s');

            $update = new Update($this->getTableGateway()->getTable());
            $update->set($data);
            $update->where([$this->columns['id'] => $id]);

            $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'SQL Statement: ' . $update->getSqlString($this->getAdapter()->getPlatform()));

            $sql       = new Sql($this->tableGateway->getAdapter());
            $statement = $sql->prepareStatementForSqlObject($update);

            $statement->execute();

            $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'The place for Slot ' . $id . ' has been set.');
            $this->getLogger()->debug($this->indent . 'Slots\Slots::savePlace() end.');

            return $this->get($id);
        } else {
            $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'Slot ID ' . $id . ' does not exists. Update failed.');
            $this->getLogger()->debug($this->indent . 'Slots\Slots::savePlace() end.');

            throw new \Exception('Slot ID does not exist');
        }
    }

    /**
     * @param Model\Slot $slot
     * @param Player     $player
     * @param array      $mixes
     */
    public function join(Model\Slot $slot, Player $player, $mixes)
    {
        $this->getLogger()->debug($this->indent . 'Slots::join() start.');

        $connection = $this->getAdapter()->getDriver()->getConnection();

        $connection->beginTransaction();

        try {

            $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'Add slot to the Slot.');

            $insert = new Insert($this->relatedTables['slots_slots']['name']);
            $insert->values([
                $this->relatedTables['slots_slots']['columns']['slot-id']   => $slot->id,
                $this->relatedTables['slots_slots']['columns']['player-id'] => $player->id,
            ]);

            $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'SQL Statement: ' . $insert->getSqlString($this->getAdapter()->getPlatform()));

            $sql       = new Sql($this->tableGateway->getAdapter());
            $statement = $sql->prepareStatementForSqlObject($insert);

            $statement->execute();

            $id = $this->getAdapter()->getDriver()->getLastGeneratedValue();

            $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'Insert mixtures made by Slot start.');

            foreach ($mixes as $mix) {
                $insert = new Insert($this->relatedTables['slots_slots_mixtures']['name']);
                $insert->values([
                    $this->relatedTables['slots_slots_mixtures']['columns']['slot-slot-id'] => $id,
                    $this->relatedTables['slots_slots_mixtures']['columns']['team-home']    => $mix['home'][0]['id'],
                    $this->relatedTables['slots_slots_mixtures']['columns']['team-away']    => $mix['away'][0]['id'],
                    $this->relatedTables['slots_slots_mixtures']['columns']['bet']          => $mix['bet'],
                ]);

                $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . self::LOGGER_INDENT . 'SQL Statement: ' . $insert->getSqlString($this->getAdapter()->getPlatform()));

                $sql       = new Sql($this->tableGateway->getAdapter());
                $statement = $sql->prepareStatementForSqlObject($insert);

                $statement->execute();
            }

            $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'Insert mixtures made by Slot end.');

            $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'Commiting transaction...');
            $connection->commit();
            $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'Transaction committed.');

        } catch (\Exception $e) {
            $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'Tossed an Exception grenade: ' . $e->getMessage());
            $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'Rolling back...');
            $connection->rollback();
            $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'Rolled back.');
        }

        $this->getLogger()->debug($this->indent . 'Slots::save() end.');
    }

    /**
     * @param Model\Slot $slot
     *
     * @throws \Exception
     */
    public function delete(Model\Slot $slot)
    {
        $this->getLogger()->debug($this->indent . 'Slots::delete() start.');

        if ($this->get($slot->id)) {
            $data = [
                $this->columns['is-active']  => 0,
                $this->columns['deleted-by'] => $slot->deletedBy,
                $this->columns['deleted-on'] => date('Y-m-d H:i:s'),
            ];

            $update = new Update($this->getTableGateway()->getTable());
            $update->set($data);
            $update->where([$this->columns['id'] => $slot->id]);

            $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'SQL Statement: ' . $update->getSqlString($this->getAdapter()->getPlatform()));

            $sql       = new Sql($this->tableGateway->getAdapter());
            $statement = $sql->prepareStatementForSqlObject($update);

            $statement->execute();

            $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'Slot ' . $slot->id . ' has been deleted.');
            $this->getLogger()->debug($this->indent . 'Slots::delete() end.');

        } else {
            $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'Slot ID ' . $slot->id . ' does not exists. Deletion failed.');
            $this->getLogger()->debug($this->indent . 'Slots::delete() end.');

            throw new \Exception('Slot ID does not exist');
        }
    }

    /**
     * Restore Slot to the table. Sets the Slot to active.
     *
     * @param Model\Slot $slot
     *
     * @throws \Exception
     */
    public function restore(Model\Slot $slot)
    {
        $this->getLogger()->debug($this->indent . 'Slots::restore() start.');

        if ($this->get($slot->id)) {
            $data = [
                $this->columns['is-active']  => 1,
                $this->columns['updated-by'] => $slot->updatedBy,
                $this->columns['updated-on'] => date('Y-m-d H:i:s'),
            ];

            $update = new Update($this->getTableGateway()->getTable());
            $update->set($data);
            $update->where([$this->columns['id'] => $slot->id]);

            $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'SQL Statement: ' . $update->getSqlString($this->getAdapter()->getPlatform()));

            $sql       = new Sql($this->tableGateway->getAdapter());
            $statement = $sql->prepareStatementForSqlObject($update);

            $statement->execute();

            $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'Slot ' . $slot->id . ' has been restored.');
            $this->getLogger()->debug($this->indent . 'Slots::restore() end.');

        } else {
            $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'Slot ID ' . $slot->id . ' does not exists. Restoration failed.');
            $this->getLogger()->debug($this->indent . 'Slots::restore() end.');

            throw new \Exception('Slot ID does not exist');
        }
    }

    /**
     * Permanently delete the Slot.
     *
     * @param Model\Slot $slot
     *
     * @throws \Exception
     */
    public function purge(Model\Slot $slot)
    {
        $this->getLogger()->debug($this->indent . 'Slots::purge() start.');

        if ($this->get($slot->id)) {
            $update = new Delete($this->getTableGateway()->getTable());
            $update->where([$this->columns['id'] => $slot->id]);

            $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'SQL Statement: ' . $update->getSqlString($this->getAdapter()->getPlatform()));

            $sql       = new Sql($this->tableGateway->getAdapter());
            $statement = $sql->prepareStatementForSqlObject($update);

            $statement->execute();

            $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'Slot ' . $slot->id . ' has been purged, as in permanently deleted.');
            $this->getLogger()->debug($this->indent . 'Slots::purge() end.');

        } else {
            $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'Slot ID ' . $slot->id . ' does not exists. Purge failed.');
            $this->getLogger()->debug($this->indent . 'Slots::purge() end.');

            throw new \Exception('Slot ID does not exist');
        }

    }

}
